Enum PgAdvisoryLockKey
#[non_exhaustive]pub enum PgAdvisoryLockKey {
BigInt(i64),
IntPair(i32, i32),
}Expand description
A key type natively used by Postgres advisory locks.
Currently, Postgres advisory locks have two different key spaces: one keyed by a single 64-bit integer, and one keyed by a pair of two 32-bit integers. The Postgres docs specify that these key spaces “do not overlap”:
https://www.postgresql.org/docs/current/functions-admin.html#FUNCTIONS-ADVISORY-LOCKS
The documentation for the pg_locks system view explains further how advisory locks
are treated in Postgres:
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
BigInt(i64)
The keyspace designated by a single 64-bit integer.
When PgAdvisoryLock is constructed with ::new(), this is the keyspace used.
IntPair(i32, i32)
The keyspace designated by two 32-bit integers.
Implementations
impl PgAdvisoryLockKey
impl PgAdvisoryLockKey