Struct PgColumn
pub struct PgColumn { /* private fields */ }Implementations
impl PgColumn
impl PgColumn
pub fn relation_id(&self) -> Option<Oid>
pub fn relation_id(&self) -> Option<Oid>
Returns the OID of the table this column is from, if applicable.
This will be None if the column is the result of an expression.
Corresponds to column attrelid of the pg_catalog.pg_attribute table:
https://www.postgresql.org/docs/current/catalog-pg-attribute.html
pub fn relation_attribute_no(&self) -> Option<i16>
pub fn relation_attribute_no(&self) -> Option<i16>
Returns the 1-based index of this column in its parent table, if applicable.
This will be None if the column is the result of an expression.
Corresponds to column attnum of the pg_catalog.pg_attribute table:
https://www.postgresql.org/docs/current/catalog-pg-attribute.html