PgDatabaseError

Struct PgDatabaseError 

pub struct PgDatabaseError(/* private fields */);
An error returned from the PostgreSQL database.

Implementations



impl PgDatabaseError

pub fn severity(&self) -> PgSeverity

pub fn code(&self) -> &str

The SQLSTATE code for this error.

pub fn message(&self) -> &str

The primary human-readable error message. This should be accurate but terse (typically one line).

pub fn detail(&self) -> Option<&str>

An optional secondary error message carrying more detail about the problem. Might run to multiple lines.

pub fn hint(&self) -> Option<&str>

An optional suggestion what to do about the problem. This is intended to differ from detail in that it offers advice (potentially inappropriate) rather than hard facts. Might run to multiple lines.

pub fn position(&self) -> Option<PgErrorPosition<'_>>

Indicates an error cursor position as an index into the original query string; or, a position into an internally generated query.

pub fn where(&self) -> Option<&str>

An indication of the context in which the error occurred. Presently this includes a call stack traceback of active procedural language functions and internally-generated queries. The trace is one entry per line, most recent first.

pub fn schema(&self) -> Option<&str>

If this error is with a specific database object, the name of the schema containing that object, if any.

pub fn table(&self) -> Option<&str>

If this error is with a specific table, the name of the table.

pub fn column(&self) -> Option<&str>

If the error is with a specific table column, the name of the column.

pub fn data_type(&self) -> Option<&str>

If the error is with a specific data type, the name of the data type.

pub fn constraint(&self) -> Option<&str>

If the error is with a specific constraint, the name of the constraint. For this purpose, indexes are constraints, even if they weren’t created with constraint syntax.

pub fn file(&self) -> Option<&str>

The file name of the source-code location where this error was reported.

pub fn line(&self) -> Option<usize>

The line number of the source-code location where this error was reported.

pub fn routine(&self) -> Option<&str>

The name of the source-code routine reporting this error.